'\" 
'\" Copyright (c) 2001 by Andreas Kupries <a.kupries@westend.com>
'\" All rights reserved.
'\" 
'\" RCS: @(#) $Id: report.n,v 1.3 2001/07/06 18:30:52 andreas_kupries Exp $
'\" 
.so man.macros
.TH report n 1.0 Report "Matrix reports"
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
::report::report \- Create and manipulate report objects
.SH SYNOPSIS
\fBpackage require report ?1.0?\fR
.sp
\fB::report::report\fR \fIreportName columns\fR ?\fBstyle\fR \fIstyle arg...\fR?
.sp
\fB::report::defstyle\fR \fIstyleName arguments script\fR
.sp
\fB::report::rmstyle\fR \fIstyleName\fR
.sp
\fB::report::stylearguments\fR \fIstyleName\fR
.sp
\fB::report::stylebody\fR \fIstyleName\fR
.sp
\fB::report::styles\fR
.sp
.BE
.SH DESCRIPTION
.PP
This package provides report objects which can be used by the
formatting methods of matrix objects to generate tabular reports of
the matrix in various forms. The report objects defined here break
each report down into three \fBregions\fR and ten classes of
\fBlines\fR (various separator- and data-lines). See the following
section for more detailed explanations.
.TP
\fB::report::report\fR \fIreportName columns\fR ?\fBstyle\fR \fIstyle
arg...\fR?  Creates a new report object for a report having
\fIcolumns\fR columns with an associated global Tcl command whose name
is \fIreportName\fR.  This command may be used to invoke various
configuration operations on the report. It has the following general
form: \fIreportName option \fR?\fIarg arg ...\fR?
.sp
\fIOption\fR and the \fIarg\fRs determine the exact behavior of the
command. See section \fBREPORT METHODS\fR for more explanations. If no
\fIstyle\fR is specified the report will use the builtin style
\fBplain\fR as its default configuration.
.TP
\fB::report::defstyle\fR \fIstyleName arguments script\fR
Defines the new style \fIstyleName\fR. See section \fBSTYLES\fR for
more information.
.TP
\fB::report::rmstyle\fR \fIstyleName\fR
Deletes the style \fIstyleName\fR. Trying to delete an unknown or
builtin style will result in an error. Beware, this command will not
check that there are no other styles depending on the deleted
one. Deleting a style which is still used by another style FOO will
result in a runtime error when FOO is applied to a newly instantiated
report.
.TP
\fB::report::stylearguments\fR \fIstyleName\fR
This introspection command returns the list of arguments associated with the
style \fIstyleName\fR.
.TP
\fB::report::stylebody\fR \fIstyleName\fR
This introspection command returns the script associated with the
style \fIstyleName\fR.
.TP
\fB::report::styles\fR
This introspection command returns a list containing the names of all
styles known to the package at the time of the call. The order of the
names in the list reflects the order in which the styles were
created. In other words, the first item is the predefined style
\fBplain\fR, followed by the first style defined by the user, and so
on.
.SH REGIONS
.PP
The three regions are the \fBtop caption\fR, \fBdata area\fR and
\fBbottom caption\fR. These are, roughly speaking, the title, the
values to report and a title at the bottom. The size of the caption
regions can be specified by the user as the number of rows they occupy
in the matrix to format. The size of the data area is specified
implicitly.
.SH LINES
.PP
Each of the ten line classes can have a \fBtemplate\fR (see section
below) associated with it defining the formatting for this kind of
line. The user is able to enable and disable the separator lines at
will, but not the data lines. Their usage is solely determined by the
number of rows contained in the three regions. Data lines and all
enabled separators must have a template associated with them.
.PP
Note that the data-lines in a report and the rows in the matrix the
report was generated from are \fBnot\fR in a 1:1 relationship if any
row in the matrix has a height greater than one.
.PP
The different kinds of lines and the codes used by the report methods
to address them are:
.TP
\fBtop\fR
The topmost line of a report. Separates the report from anything which
came before it. The user can enable the usage of this line at will.
.TP
\fBtopdatasep\fR
This line is used to separate the data rows in the top caption region,
if it contains more than one row and the user enabled its usage.
.TP
\fBtopcapsep\fR
This line is used to separate the top caption and data regions, if the
top caption is not empty and the user enabled its usage.
.TP
\fBdatasep\fR
This line is used to separate the data rows in the data region, if it
contains more than one row and the user enabled its usage.
.TP
\fBbotcapsep\fR
This line is used to separate the data and bottom caption regions, if
the bottom caption is not empty and the user enabled its usage.
.TP
\fBbotdatasep\fR
This line is used to separate the data rows in the bottom caption
region, if it contains more than one row and the user enabled its
usage.
.TP
\fBbottom\fR
The bottommost line of a report. Separates the report from anything
which comes after it. The user can enable the usage of this line at
will.
.TP
\fBtopdata\fR
This line defines the format of data lines in the top caption region
of the report.
.TP
\fBdata\fR
This line defines the format of data lines in the data region of the
report.
.TP
\fBbotdata\fR
This line defines the format of data lines in the bottom caption
region of the report.
.SH TEMPLATES
.PP
Each template is a list of strings used to format the line it is
associated with. For a report containing \fIn\fR columns a template
for a data line has to contain "\fIn\fR+1" items and a template for a
separator line "2*\fIn\fR+1" items.
.PP
The items in a data template specify the strings used to separate the
column information. Together with the corresponding items in the
separator templates they form the vertical lines in the
report.
.PP
\fBNote\fR that the corresponding items in all defined templates have
to be of equal length. This will be checked by the report object. The
first item defines the leftmost vertical line and the last item
defines the rightmost vertical line. The item at index \fIk\fR
("1",...,"\fIn\fR-2") separates the information in the columns
"\fIk\fR-1" and "\fIk\fR".
.PP
The items in a separator template having an even-numbered index
("0","2",...)  specify the column separators. The item at index
"2*\fIk\fR" ("0","2",...,"2*\fIn\fR") corresponds to the items at
index "\fIk\fR" in the data templates.
.PP
The items in a separator template having an odd-numbered index
("1","3",...) specify the strings used to form the horizontal lines in
the separator lines. The item at index "2*\fIk\fR+1"
("1","3",...,"2*\fIn\fR+1") corresponds to column "\fIk\fR". When
generating the horizontal lines the items are replicated to be at
least as long as the size of their column and then cut to the exact
size.
.SH STYLES
.PP
Styles are a way for the user of this package to define common
configurations for report objects and then use them later during the
actual instantiation of report objects. They are defined as tcl
scripts which when executed configure the report object into the
requested configuration.
.PP
The command to define styles is \fB::report::defstyle\fR. Its last
argument is the tcl \fIscript\fR performing the actual reconfiguration
of the report object to obtain the requested style.
.PP
In this script the names of all previously defined styles are
available as commands, as are all commands found in a safe interpreter
and the configuration methods of report objects. The latter implicitly
operate on the object currently executing the style script. The
\fIarguments\fR declared here are available in the \fIscript\fR as
variables. When calling the command of a previously declared style all
the arguments expected by it have to be defined in the call.
.SH REPORT METHODS
.PP
The following commands are possible for report objects:
.TP
\fIreportName\fR \fBdestroy\fR
Destroys the report, including its storage space and associated
command.
.TP
\fIreportName\fR \fItemplatecode\fR \fBdisable\fR|\fBenable\fR
Enables or disables the usage of the template addressed by the
\fItemplatecode\fR. Only the codes for separator lines are allowed
here. It is not possible to enable or disable data lines.
.sp
Enabling a template causes the report to check all used templates for
inconsistencies in the definition of the vertical lines (See section
\fBTEMPLATES\fR).
.TP
\fIreportName\fR \fItemplatecode\fR \fBenabled\fR
Returns the whether the template addressed by the \fItemplatecode\fR is
currently enabled or not.
.TP
\fIreportName\fR \fItemplatecode\fR \fBget\fR
Returns the template currently associated with the kind of line
addressed by the \fItemplatecode\fR. All known templatecodes are
allowed here.
.TP
\fIreportName\fR \fItemplatecode\fR \fBset\fR \fItemplatedata\fR
Sets the template associated with the kind of line addressed by the
\fItemplatecode\fR to the new value in \fItemplatedata\fR. See section
\fBTEMPLATES\fR for constraints on the length of templates.
.TP
\fIreportName\fR \fBtcaption\fR ?\fIsize\fR?
Specifies the \fIsize\fR of the top caption region as the number rows
it occupies in the matrix to be formatted. Only numbers greater than
or equal to zero are allowed. If no \fIsize\fR is specified the
command will return the current size instead.
.sp
Setting the size of the top caption to a value greater than zero
enables the corresponding data template and causes the report to check
all used templates for inconsistencies in the definition of the
vertical lines (See section \fBTEMPLATES\fR).
.TP
\fIreportName\fR \fBbcaption\fR \fIsize\fR
Specifies the \fIsize\fR of the bottom caption region as the number
rows it occupies in the matrix to be formatted. Only numbers greater
than or equal to zero are allowed. If no \fIsize\fR is specified the
command will return the current size instead.
.sp
Setting the size of the bottom caption to a value greater than zero
enables the corresponding data template and causes the report to check
all used templates for inconsistencies in the definition of the
vertical lines (See section \fBTEMPLATES\fR).
.TP
\fIreportName\fR \fBsize\fR \fIcolumn\fR ?\fInumber\fR|\fBdyn\fR?
Specifies the size of the \fIcolumn\fR in the output. The value
\fBdyn\fR means that the columnwidth returned by the matrix to be
formatted for the specified column shall be used. The formatting of
the column is "dynamic". If a fixed \fInumber\fR is used instead of
\fBdyn\fR it means that the column has a width of that many characters
(padding excluded). Only numbers greater than zero are allowed here.
.sp
If no size specification is given the command will return the current
size of the \fIcolumn\fR instead.
.TP
\fIreportName\fR \fBsizes\fR ?\fIsize-list\fR?
This method allows the user to specify the sizes of all columns in one
call. Its argument is a list containing the sizes to associate with
the columns. The first item is associated with column 0, the next with
column 1, and so on.
.sp
If no \fIsize-list\fR is specified the command will return a list
containing the currently set sizes instead.
.TP
\fIreportName\fR \fBpad\fR \fIcolumn\fR ?\fBleft\fR|\fBright\fR|\fBboth\fR ?\fIpadstring\fR??
This method allows the user to specify padding on the left, right or
both sides of a \fIcolumn\fR. If the \fIpadstring\fR is not specified
it defaults to a single space character. \fBNote\fR: An alternative
way of specifying the padding is to use vertical separator strings
longer than one character in the templates (See section
\fBTEMPLATES\fR).
.sp
If no pad specification is given at all the command will return the
current state of padding for the column instead. This will be a list
containing two elements, the first element the left padding, the
second describing the right padding.
.TP
\fIreportName\fR \fBjustify\fR \fIcolumn\fR ?\fBleft\fR|\fBright\fR|\fBcenter\fR?
Declares how the cell values for a \fIcolumn\fR are filled into the
report given the specified size of a column in the report.
.sp
For \fBleft\fR and \fBright\fR justification a cell value shorter than
the width of the column is bound with its named edge to the same edge
of the column. The other side is filled with spaces. In the case of
\fBcenter\fR the spaces are placed to both sides of the value and the
left number of spaces is at most one higher than the right number of
spaces.
.sp
For a value longer than the width of the column the value is cut at
the named edge. This means for \fBleft\fR justification that the
\fBtail\fR (i.e. the \fBright\fR part) of the value is made visible in
the output. For \fBcenter\fR the value is cut at both sides to fit
into the column and the number of characters cut at the left side of
the value is at most one less than the number of characters cut from
the right side.
.sp
If no justification was specified the command will return the
current justification for the column instead.
.TP
\fIreportName\fR \fBprintmatrix\fR \fImatrix\fR
Formats the \fImatrix\fR according to the configuration of the report
and returns the resulting string. The matrix has to have the same
number of columns as the report. The matrix also has to have enough
rows so that the top and bottom caption regions do not overlap. The
data region is allowed to be empty.
.TP
\fIreportName\fR \fBprintmatrix2channel\fR \fImatrix chan\fR
Formats the \fImatrix\fR according to the configuration of the report
and writes the result into the channel \fIchan\fR. The matrix has to
have the same number of columns as the report. The matrix also has to
have enough rows so that the top and bottom caption regions do not
overlap. The data region is allowed to be empty.
.TP
\fIreportName\fR \fBcolumns\fR
Returns the number of columns in the report.
.PP
The methods \fBsize\fR, \fBpad\fR and \fBjustify\fR all take a column
index as their first argument. This index is allowed to use all the
forms of an index as accepted by the \fBlindex\fR command. The allowed
range for indices is "0,...,[\fIreportName\fR columns]-1".
.SH EXAMPLES
.PP
Our examples define some generally useful report styles.
.PP
A simple table with lines surrounding all information and vertical
separators, but without internal horizontal separators.
.PP
.CS
::report::defstyle simpletable {} {
	data	set [split "[string repeat "| "   [columns]]|"]
	top	set [split "[string repeat "+ - " [columns]]+"]
	bottom	set [top get]
	top	enable
	bottom	enable
}
.CE
.PP
An extension of a \fBsimpletable\fR, see above, with a title area.
.PP
.CS
::report::defstyle captionedtable {{n 1}} {
	simpletable
	topcapsep set [top get]
	topcapsep enable
	tcaption $n
}
.CE
.SH KEYWORDS
matrix, report, table
